package mw;

import java.util.*;
//import pnp.persistent.*;

/**
 *
 * Class Delta:
 * This class models a DELTA data provider.
 *
 * @author Noel Cross
 * @author Michael Bartley
 */
public class Delta implements DeltaInterface, Persistent {
    

    /*************************************************************************
     *
     * Delta
     *     public constructor
     * empty body, no arguments taken
     *
     ************************************************************************/
    public Delta(){
    }
    


    /*************************************************************************
     *
     * getItemCharacterList
     *    public method
     * input:   Vector  should hold elements of BasicState objects
     *          boolean sets use of intelligent aspects
     * output:  Vector  contains elements of BasicItemCharacter objects
     *
     ************************************************************************/
    public Vector getItemCharacterList(Vector basicStates, boolean intelligent){
	Vector v = new Vector();
	
	//SHRINKING LOGIC REMOVED FOR SPEED ISSUES
	//WILL BE RE-IMPLEMENTED AT A LATER DATE AND TIME
	//MIKE, 2/9/99
	//PUT BACK IN, MIKE -- 6/7/99
	if(intelligent && basicStates != null){
	    //if(false){
	    Hashtable charsUsed = new Hashtable();  //key = state, object = char
	    //first put all selected characters in a lookup hash
	    //anything already used will not be offered for future selections
	    for(int i=0; i<basicStates.size(); i++){
		State s = State.getState((BasicState)basicStates.elementAt(i));
		charsUsed.put(s, s.getItemCharacter());
	    }
	    Vector items = getItemList(basicStates);
	    if(items.size() > 1){
		Enumeration allChars = ItemCharacter.elements();
		Hashtable chars = new Hashtable();  //key = state, object = char
		while(allChars.hasMoreElements()){
		    ItemCharacter ic = (ItemCharacter)allChars.nextElement();
		    if(charsUsed.contains(ic))
			continue;
		    //note, we haven't checked to see if this will actually be
		    //a useful state, only that its a number and used by at 
		    //least two items.  Ergo this call can be improved.
		    
		    else if(ic.isNumeric()){
			int count = 0;
			for(int i=0; i<items.size(); i++){
			    Attribute a = ((Item)items.elementAt(i)).getAttributeWithItemCharacter(ic);
			    if(a != null)
				count++;
			    if(count > 1){
				v.addElement(ic);
				break;
			    }
			}
			continue;
		    }
		    
		    //don't show text characters as choices
		    else if(ic.isText())
			continue;
		    
		    //got here, so must be multi-state
		    int stateCount = 0;
		count_states:
		    for(int i=0; i<items.size(); i++){
			Attribute a = ((Item)items.elementAt(i)).getAttributeWithItemCharacter(ic);
			//if we have an attribute, check it for states
			if(a != null){
			    Vector states = a.getStates();
			    for(int j=0; j<states.size(); j++){
				State s = (State)states.elementAt(j);
				//if a state hasn't been used to id a char 
				//before, add it to hash and up state count.  
				//The hash is neccessary so we don't up
				//state count on the same state.
				
				if(!chars.containsKey(s)){
				    chars.put(s, ic);
				    stateCount++;
				}
			    }
			    
			    /*we have multiple choices, so char is important
			      NOTE that we haven't checked to see if the state
			      will reduce our items list, only if multiple 
			      states are selected.  i.e., we could have two 
			      items, one has this character and two state 
			      values, the other doesn't have this character, 
			      then this character will be listed along with 
			      two states, but no selection will reduce item 
			      list.  Ergo this call can be improved.
			    */
			    if(stateCount > 1){
				v.addElement(ic);
				//should end for loop, since we've 
				//justified adding a character
				break count_states; 
			    }
			}
		    }
		}
	    }
	}
	//END OF SHRINKING LOGIC
	else{
	    //System.out.println("\nNO BASIC STATES TO USE FOR SHRINKING!\n");
	    Enumeration e = ItemCharacter.elements();
	    while(e.hasMoreElements()){
		BasicItemCharacter ic = (ItemCharacter)e.nextElement();
		if(!ic.isText())
		    v.addElement(ic);
	    }
	}
	return Sorter.sortVector(v);
    }


    /*************************************************************************
     *
     * getStateList
     *    public method
     * input:   Vector, Object;  Vector should contain elements of type
     *                    BasicState, Object should be an Integer
     *          boolean  identifies use of intelligent features
     * output:  Vector   will contain elements of type BasicState, null
     *                    if Object does not identify a multi-state character
     *
     ************************************************************************/
    public Vector getStateList(Vector basicStates, Object characterId, boolean intelligent){
	Vector v = new Vector();
	ItemCharacter ic = ItemCharacter.restore((Integer)characterId);
	if(!ic.isMultiState())
	    return null;
	
	//SHRINKING LOGIC REMOVED FOR SPEED ISSUES
	//WILL BE RE-IMPLEMENTED AT A LATER DATE AND TIME
	//MIKE, 2/9/99
	//PUT BACK IN, MIKE -- 6/7/99
	//if(false){
	if(intelligent && basicStates != null){
	    //key = state, object = char
	    Hashtable charsUsed = new Hashtable();  
	    for(int i=0; i<basicStates.size(); i++){
		State s =State.getState((BasicState)basicStates.elementAt(i));
		charsUsed.put(s, s.getItemCharacter());
	    }
	    Vector items = getItemList(basicStates);
	    for(int i=0; i<items.size(); i++){
		Attribute a = ((Item)items.elementAt(i)).getAttributeWithItemCharacter(ic);
		if(a != null){
		    Vector states = a.getStates();
		    for(int j=0; j<states.size(); j++){
			State s = (State)states.elementAt(j);
			if(!v.contains(s))
			    v.addElement(s);
		    }
		}
	    }
	}
	//END SHRINKING LOGIC
	else{
	    Vector states = ic.getStates();
	    for(int i=0; i<states.size(); i++){
		BasicState bs = (BasicState)states.elementAt(i);
		v.addElement(bs);
	    }
	}
	return v;
    }

    
    /*************************************************************************
     *
     * getItemList
     *    public method
     * input:   Vector   should contain elements of type BasicState
     * output:  Vector   contains elements of type BasicItem
     *
     ************************************************************************/
    public Vector getItemList(Vector basicStates){
	Vector v = new Vector();
	Enumeration e = Item.elements();
	int i=0;
	while(e.hasMoreElements()){
	    //System.out.println("Added element number " + ++i);
	    v.addElement((Item)e.nextElement());
	}
	//System.out.println("Done generating item list");
	if(basicStates != null){
	    Item item = buildItem(basicStates);
	    v = getBasicPossibleMatches(item, v);
	}
	//Vector rv = new Vector();
	//for(i=0; i<v.size(); i++){
	//  rv.addElement((BasicItem)v.elementAt(i));
	//}
	return Sorter.sortVector(v);
    }
    

    /*************************************************************************
     *
     * buildItem
     *     private method
     * input:   Vector   should contain elements of type BasicState
     * output:  Item     item based on give states
     *
     ************************************************************************/
    private Item buildItem(Vector basicStates){
	Item item = new Item();
	for(int i=0; i<basicStates.size(); i++){
	    //NOTE: could redo Attribute.java and its subclasses to make this nicer
	    // already have State.getState(BasicState bs) to help!
	    BasicState bs = (BasicState)basicStates.elementAt(i);
	    ItemCharacter ic = ItemCharacter.restore(bs.getCharacterId());
	    if(ic.isMultiState()){
		Vector states = ic.getStates();
		for(int j=0; j<states.size(); j++){
		    State s = (State)states.elementAt(j);
		    if(s.getName().compareTo(bs.getName()) == 0){
			Attribute a = new MultiStateAttribute();
			a.setItemCharacter(ic);
			a.addCommentedState(s, "");
			item.addAttribute(a);
			break;
		    }
		}
	    }
	    else if(ic.isNumeric()){
		Attribute a = new NumericAttribute();
		a.setItemCharacter(ic);
		NumericState ns = new NumericState(bs);
		ns.setItemCharacter(ItemCharacter.restore(bs.getCharacterId()));
		a.addCommentedState(ns, "");
		item.addAttribute(a);
	    }
	}
	return item;
    }
    

    /*************************************************************************
     * 
     * getPossibleMatches
     *   input:   Item, Vector;  Vector should contain all possible items to 
     *                           be compared against Item
     *   output:  Vector  -  contains all possible Item matches
     *
     ************************************************************************/
    private Vector getPossibleMatches(Item item, Vector allItems){
	Vector v = new Vector();
	for(int i = 0; i < allItems.size(); i++){
	    Item item2 = (Item)allItems.elementAt(i);
	    //CHANGED FOR SPEED -- MIKE, 6/7/99
	    //if(!item2.cannotBe(item)){
	    if(!item.cannotBe(item2)){
		v.addElement(item2);
		//System.out.println("eliminated " + i);
	    }
	    //else{
	    //v.addElement(item2);
	    //}
	}
	System.out.println("returning " + v.size() + "items");
	return v;
    }

    private Vector getBasicPossibleMatches(Item item, Vector allItems){
	Vector v = new Vector();
	for(int i = 0; i < allItems.size(); i++){
	    Item item2 = (Item)allItems.elementAt(i);
	    //CHANGED FOR SPEED -- MIKE, 6/7/99
	    //if(!item2.cannotBe(item)){
	    if(!item.cannotBe(item2)){
		v.addElement((BasicItem)item2);
		//System.out.println("eliminated " + i);
	    }
	    //else{
	    //v.addElement(item2);
	    //}
	}
	System.out.println("returning " + v.size() + "items");
	return v;
    }
    


    /*************************************************************************
     *
     * getItem
     *   input:   Object  - should be identifying integer
     *   output:  Item  -  item with id of Object
     *
     ************************************************************************/
    public Item getItem(Object itemId){
	Item i = Item.restore((Integer)itemId);
	return i;
    }
    

    /*************************************************************************
     *
     * getItemTitle
     *   input:   Object  - should be identifying integer
     *   output:  String  - name of identified item
     *
     ************************************************************************/
    public String getItemTitle(Object itemId){
	return getItem(itemId).getName();
    }
    

    /*************************************************************************
     *
     * getItemDescription
     *   input:   Object  - should be identifying integer
     *   output:  Vector  - contains set of strings describing item
     *
     ************************************************************************/
    public Vector getItemDescription(Object itemId){
	Item myItem = getItem(itemId);
	Vector strings = new Vector();
	String currentString;
	Vector atv = myItem.getAttributes();
	for(int i = 0; i < atv.size(); i++){
	    Attribute at = (Attribute)atv.elementAt(i);
	    currentString = at.getComment();
	    if(currentString == null)
		currentString = "";
	    currentString =currentString + at.getItemCharacter().getFeature();
	    if(at.getItemCharacter().isMultiState()){
		Vector csv = at.getCommentedStates();
		for(int j = 0; j < csv.size(); j++){
		    CommentedState cs = (CommentedState)csv.elementAt(j);
		    State s = cs.getState();
		    String sn;
		    try{
			sn = s.getName();
		    }
		    catch(Exception e){
			sn = "";
			System.out.println("OOPS!" + e);
		    }
		    String sc = cs.getComment();
		    if(sc == null)
			sc = "";
		    currentString = currentString +":  "+ sn + sc;
		}
	    }
	    else if(at.getItemCharacter().isNumeric()){
		NumericAttribute na = (NumericAttribute)at;
		String rs;
		try{
		    rs = na.shortOutput();
		}
		catch(Exception e){
		    rs = "NUMBER NOT SHOWING UP";
		}
		currentString = currentString + ":  " + rs;
	    }
	    else if (at instanceof TextAttribute){
		try{
		    TextAttribute ta = (TextAttribute)at;
		    TextState ts = ta.getTextState();
		    currentString = currentString + ":  " + ts.getValue();
		}
		catch(Exception e){
		    System.out.println("Must be a resource.");
		}
	    }
	    strings.addElement(currentString);
	}
	return strings;
    }


    /*************************************************************************
     *
     * getItemResources
     *   input:   Object  - should be identifying integer
     *   output:  Vector  - elements are of type Resource of requested Item
     *
     ************************************************************************/
    public Vector getItemResources(Object itemId){
	Vector rv = new Vector();
	Item myItem = getItem(itemId);
	Vector atv = myItem.getAttributes();
	String title = myItem.getName();
	for(int i = 0; i < atv.size(); i++){
	    Attribute at = (Attribute)atv.elementAt(i);
	    if(at instanceof ResourceAttribute){
		try{
		    ResourceAttribute ra = (ResourceAttribute)at;
		    Vector v = ra.getResources();
		    for(int j=0; j<v.size(); j++){
			try{
			    ResourceState rs = (ResourceState)v.elementAt(j);
			    Resource re= rs.getResource();
			    re.setTitle(title);
			    rv.addElement(re);
			}
			catch(Exception e){
			    System.out.println("Unable to get resource. "+e);
			}
		    }
		}
		catch(Exception e){
		    System.out.println("Not a resource, must be text. "+e);
		}
	    }
	}
	return rv;
    }
    

    /*************************************************************************
     *
     * getBasicItemCharacter
     *   input:   Object  - should be indentifying integer
     *   output:  BasicItemCharacter
     ************************************************************************/
    public BasicItemCharacter getBasicItemCharacter(Object itemCharacterId){
	return (BasicItemCharacter)ItemCharacter.restore((Integer)itemCharacterId);
    }
    

    //########################################################################
    // Persistence stuff
    //########################################################################

    public Object getId(){
	return new Integer(0);
    }
    public void setId(Object o){
    }
    private static PersistentPeer thePeer;
    public static void setPersistentPeer(PersistentPeer peer) {thePeer = peer;}
    public PersistentPeer getPersistentPeer() {return thePeer;}
    
    public void restore() {
	try{
	    thePeer.restore(this);
	}catch(NothingRestoredException e){}
    }
    public void update() {thePeer.update(this);}
    public void insert() {thePeer.insert(this);}
    public void delete() {thePeer.delete(this);}
}

