package mw;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
//import pnp.persistent.*;

/**
   
 * This class models a DELTA Item.
 *
 * @author Noel Cross
 * @author Michael Bartley
 */

public class Item extends BasicItem implements Persistent {
    private Vector attributes;
    
    private static Hashtable cache = new Hashtable();
    
    public void setId(Object id){
	this.id = (Integer)id;
	if(id != null){
	    cache.put(this.id, this);
	}
    }
    
    public Item(){
	super();
	attributes = new Vector();
    }
    
    public void setAttributes(Vector attributes){
	this.attributes = attributes;
    }
    
    /** 
	Returns true if a given Item has no states that contradict mine.
	This includes
    */
    public boolean cannotBe(Item otherItem){
	
	//assume we could be the same until contradicted
	//loop though all my attributes
	//REVERSED COMPARISON ORDERS FOR SPEED GAINS -- MIKE 6/7/99
	for(int i = 0; i < attributes.size(); i++){
	    Attribute thisAttrib = (Attribute)attributes.elementAt(i);
	    
	    //if this otherItem has this attribute, get the attribute
	    ItemCharacter ic = thisAttrib.getItemCharacter();
	    if(otherItem.containsItemCharacter(ic)){
		Attribute theOtherAttrib = otherItem.getAttributeWithItemCharacter(ic);
		//if(!thisAttrib.sameAs(theOtherAttrib)){
		if(!theOtherAttrib.sameAs(thisAttrib)){
		    //can't be the same item
		    return true;
		}
	    }
	}
	return false;
    }
    
    public Vector getAttributes(){
	return attributes;
    }
    
    public void addAttribute(Attribute a){
	if(a !=  null){
	    attributes.addElement(a);
	}
    }
    
    /**
     * Compares the current item and a new item
     * returning all distinguishing ItemCharacters.
     */
    /* NOT USED
       public Vector compareTo(Item itemB){
       Vector v = new Vector();
       
       //loop through my attributes
       for(int i = 0; i < attributes.size(); i++){
       Attribute attribA = (Attribute)attributes.elementAt(i);
       ItemCharacter icA = attribA.getItemCharacter();
       
       //if a character is not multistate, add it to returnv
       if(!(icA.isMultiState())){
       v.addElement(icA);
       }
       else{
       // if itemB doesn't share the character, add it to returnv
       if(!itemB.containsItemCharacter(icA)){
       v.addElement(icA);
       }
       
       // if we share a multistate character but have different states for it
       // add it to the vector
       boolean hasMultiState = false;
       
       Vector statesA = attribA.getStates();
       
       if(statesA != null && statesA.size() > 0){
       System.out.print("stateA has " + statesA.size() + " elements.");
       for(int j = 0; j < statesA.size(); j++){
       State stateA = (State)statesA.elementAt(j);
       System.out.println(stateA);
       if(itemB.containsState(stateA)){
       hasMultiState = true;
       }
       
       if(!hasMultiState){
       v.addElement(icA);
       }
       }
       }
       }
       }
       return v;
       }
    */
    
    public Attribute getAttributeWithState(State st){
	int size = attributes.size();
	for(int i = 0; i < size; i++){
	    Attribute a = (Attribute)attributes.elementAt(i);
	    if(a.containsState(st)){
		return a;
	    }
	}
	return null;
    }

    public Attribute getAttributeWithItemCharacter(ItemCharacter ic){
	int size = attributes.size();
	for(int i = 0; i < size; i++){
	    Attribute a = (Attribute)attributes.elementAt(i);
	    ItemCharacter ic2 = a.getItemCharacter();
	    if(ic == ic2){
		return a;
	    }
	}
	return null;
    }

    public boolean hasAttribute(Attribute a){
	return attributes.contains(a);
    }

    public boolean containsState(State s){
	int size = attributes.size();
	for(int i = 0; i < size; i++){
	    Attribute a = (Attribute)attributes.elementAt(i);
	    if(a.containsState(s)){
		return true;
	    }
	}
	return false;
    }

    public Vector getItemCharacters(){
	Vector v = new Vector();
	int size = attributes.size();
	for(int i = 0; i < size; i++){
	    Attribute a = (Attribute)attributes.elementAt(i);
	    v.addElement(a.getItemCharacter());
	}
	return v;
    }

    public boolean containsItemCharacter(ItemCharacter ic){
	return getItemCharacters().contains(ic);
    }

    public void clearAttributes(){
	attributes.removeAllElements();
    }

    public void print(){
	System.out.println(toString());
    }

    public String toString(){
	StringBuffer sb = new StringBuffer();
	sb.append(name);
	for(int i = 0; i < attributes.size(); i++) {
	    Attribute a = (Attribute)attributes.elementAt(i);
	    sb.append(a.toString());
	}
	return new String(sb);
    }
    
    // Persistence stuff
    private static PersistentPeer thePeer;
    public static void setPersistentPeer(PersistentPeer peer){thePeer = peer;}
    public PersistentPeer getPersistentPeer() {return thePeer;}
    
    public static Item restore(Integer id) {
	Item item = (Item)cache.get(id);
	if(item == null){
	    item = new Item();
	    item.setId(id);
	    item.restore();
	}
	//try again.
	item = (Item)cache.get(id);
	return item;
    }
    public static void restoreAll(){
	System.out.println("Start restoreAll");
	Item item = new Item();
	item.setId(null);
	item.restore();
	System.out.println("End restoreAll");
    }
    public static Enumeration elements(){
	return cache.elements();
    }
    public void restore() {
	try{
	    thePeer.restore(this);
	}catch(NothingRestoredException e){}
    }
    public void update() {thePeer.update(this);}
    public void insert() {thePeer.insert(this);}
    public void delete() {thePeer.delete(this);}
    
}
