package mw;
import java.util.*;

/**
 * @author Noel Cross 
 * @author Michael Bartley
 */
public class NumericState extends State {
  static Double max;
  static Double min;
  static Double extremeMax;
  static Double extremeMin;

  boolean isReal;

  double upper;
  double lower;
  double median;
  double extremeUpper;
  double extremeLower;

  public NumericState() {
    super();
    min = null;
    max = null;
    extremeMin = null;
    extremeMax = null;
  }

  public NumericState(BasicState bs){
    super();
    setCharacterId(bs.getCharacterId());
    setItemCharacter(ItemCharacter.restore(characterId));

    String str = bs.getName();

    StringBuffer sb = new StringBuffer();

    //filter out unwanted characters
    for(int i = 0; i < str.length(); i++) {
      char c = str.charAt(i);
      if(c == '(' ||  c == ')' || c == '-' || c == '.' || isDigit(c)){
	sb.append(c);
      }
    }
    //if nothing is left then get out of here.
    if(sb.length() == 0){
      return;
    }
    //create a position marker
    int position = 0;

    StringBuffer extremeLower = new StringBuffer();
    //if there's a lower extreme range
    if(sb.charAt(0) == '(') {
      position++; //move ahead
      //keep going until there's a close parantheses
      while(sb.charAt(position) != ')' && position < sb.length()){
	char c = sb.charAt(position);
	if(isDigit(c) || c == '.'){
	  extremeLower.append(c);
	}
	position++;
      }
    }

    // set the value for extreme lower
    if(extremeLower.length() > 0){
      Double d = new Double(new String(extremeLower));
      setExtremeLower(d.doubleValue());
    }

    //move past the closed paretheses, if it exists
    if(sb.charAt(position) == ')') {
      position++; //move ahead
    }

    //now get lower-median-upper set.

    StringBuffer lowerMedianUpper = new StringBuffer();

    while(position < sb.length() && sb.charAt(position) != '('){
      lowerMedianUpper.append(sb.charAt(position));
      position++;
    }
    //split into lower, median, upper
    StringTokenizer st = new StringTokenizer(new String(lowerMedianUpper), "-");
    int count = 0;
    String typeAbbrev = new String();
    Vector values = new Vector();
    while(st.hasMoreTokens()){
      count++;
      values.addElement(new String(st.nextToken()));
    }
    if(values.size() == 1){
      Double d = new Double((String)values.elementAt(0));
      set(d.doubleValue());
    }
    else if(values.size() == 2){
      Double d = new Double((String)values.elementAt(0));
      setLower(d.doubleValue());

      d = new Double((String)values.elementAt(1));
      setUpper(d.doubleValue());
    }
    else if(values.size() == 3){

      Double d = new Double((String)values.elementAt(0));
      setLower(d.doubleValue());

      d = new Double((String)values.elementAt(1));
      setMedian(d.doubleValue());

      d = new Double((String)values.elementAt(2));
      setUpper(d.doubleValue());
    }

    StringBuffer extremeUpper = new StringBuffer();
    while(position < sb.length() && sb.charAt(position) != ')'){
      char c = sb.charAt(position);
      if(isDigit(c) || c == '.'){
	extremeUpper.append(c);
      }
      position++;
    }
    // set the value for extreme upper
    if(extremeUpper.length() > 0){
      Double d = new Double(new String(extremeUpper));
      setExtremeUpper(d.doubleValue());
    }
  }

  public boolean containedIn(Vector v){
    for(int i = 0; i < v.size(); i++){
      NumericState ns = (NumericState)v.elementAt(i);
      if(fallsWithinExtremeRange(ns)) {
	return true;
      }
    }
    return false;
  }

  public void set(double d){
    this.upper = d;
    this.lower = d;
    median = d;
    setMax(d);
    setMin(d);
  }
  public void setLower(double d){
    this.lower = d;
    setMin(d);
  }
  public void setUpper(double d){
    this.upper = d;
    setMax(d);
  }
  public void setMedian(double d){
    this.median = d;
  }
  public void setExtremeLower(double d){
    this.extremeLower = d;
    setExtremeMin(d);
  }
  public void setExtremeUpper(double d){
    this.extremeUpper = d;
    setExtremeMax(d);
  }
  public double getMedian(){return median;}
  public double getLower(){return lower;}
  public double getUpper(){return upper;}
  public Double getMin(){return min;}
  public Double getMax(){return max;}
  public double getExtremeLower(){return extremeLower;}
  public double getExtremeUpper(){return extremeUpper;}
  public Double getExtremeMin(){return extremeMin;}
  public Double getExtremeMax(){return extremeMax;}

  public void setIsReal(){isReal = true;}
  public void setIsInteger(){isReal = false;}

  private void setMin(double d){
    if(min == null){
      min = new Double(d);
    }
    else if (d < min.doubleValue()){
      min = new Double(d);
    }
  }
  private void setMax(double d){
    if(max == null){
      max = new Double(d);
    }
    else if (d > max.doubleValue()){
      max = new Double(d);
    }
  }
  private void setExtremeMin(double d){
    if(extremeMin == null){
      extremeMin = new Double(d);
    }
    else if (d < extremeMin.doubleValue()){
      extremeMin = new Double(d);
    }
  }
  private void setExtremeMax(double d){
    if(extremeMax == null){
      extremeMax = new Double(d);
    }
    else if (d > extremeMax.doubleValue()){
      extremeMax = new Double(d);
    }
  }
  public boolean fallsWithinNormalRange(NumericState s){
    if(s.getUpper() <= upper && s.getLower() >= lower) {
      return true;
    }
    else {
      return false;
    }
  }
  public boolean fallsWithinExtremeRange(NumericState s){
    if(s.getUpper() == s.getLower()){
      if(s.getMedian()  == median)
	return true;
      if(s.getMedian() <= upper && s.getMedian() >= lower)
	return true;
      if(s.getMedian() <= extremeUpper && s.getMedian() >= extremeLower)
	return true;
    }
    if(s.getUpper() <= upper && s.getLower() >= lower)
      return true;
    if(s.getUpper() <= extremeUpper && s.getLower() >= extremeLower)
      return true;
    if(s.getExtremeUpper() != s.getExtremeLower() &&
       s.getExtremeUpper() <= extremeUpper && 
       s.getExtremeLower() >= extremeLower) 
      return true;
    return false;
  }

  //Returns a string representation for the values of this state
  public String getStringValue() {
    StringBuffer sb = new StringBuffer();
    if(lower == upper){
      sb.append(median);
    }
    else{
      if(extremeLower != 0){
	sb.append("(" + extremeLower + "-)");
      }
      if(lower != 0){
	sb.append(lower);
      }
      if(median != 0){
	sb.append("-" + median);
      }
      if(upper != 0){
	sb.append("-" + upper);
      }
      if(extremeUpper != 0){
	sb.append("(-" + extremeUpper + ")");
      }
    }
    return new String(sb);
  }
  public void print(){
    System.out.println(toString());
  }
  public String toString(){
    StringBuffer sb = new StringBuffer();
    sb.append("name: " + getName());
    sb.append("upper: " + upper);
    sb.append("lower: " + lower);
    sb.append("median: " + median);
    if(max != null){
      sb.append("max: " + max);
    }
    else if(min != null){
      sb.append("min: " + min);
    }
    sb.append("extremeUpper: " + extremeUpper);
    sb.append("extremeLower: " + extremeLower);

    if(extremeMax != null){
      sb.append("extremeMax: " + extremeMax);
    }
    else if(extremeMin != null){
      sb.append("extremeMin: " + extremeMin);
    }
    return new String(sb);
  }
  public static void main(String args[]){

    NumericState ns1 = new NumericState();
    ns1.setName("mm");
    ns1.setUpper(7.6);
    ns1.setMedian(5);
    ns1.setLower(4.2);
    ns1.print();
    System.out.println("-----");

    NumericState ns2 = new NumericState();
    ns2.setName("mm");
    ns2.setUpper(8.6);
    ns2.setLower(3.9);
    ns2.print();
    System.out.println("-----");
    if(ns2.fallsWithinNormalRange(ns1)){
      System.out.println("ns2 falls within ns1");
    }
    if(ns1.fallsWithinNormalRange(ns2)){
      System.out.println("ns1 falls within ns2");
    }
    System.out.println(ns1.getStringValue());
  }

  private boolean isDigit(char c){
    if(c>'0'-1 && c<'9'+1)
      return true;
    return false;
  }

}
