package mw;

import java.io.*;
//import java.text.*;
import java.util.Vector;
import java.util.StringTokenizer;

/**
 * <pre>
 * This class reads a DELTA chars file as described in
 * Definition of the Delta Format
 * 14 March, 1995
 * Dallwitz & Paine 
 * </pre>
 * @author Noel Cross
 * @author Michael Bartley
 */

public class CharsFileReader {
  DataInputStream charsFile;
  Vector itemCharacters;

  //this vector comes from parsing a Specs file with the SpecFileReader class.
  Vector itemCharacterTypes;

  public CharsFileReader(){
    itemCharacters = new Vector();
  }
  //mandatory!
  public void setItemCharacterTypes(Vector v){
    itemCharacterTypes = v;
  }

  public String lookupItemCharacterTypeAbbrev(Object o) {
    if(o instanceof Integer){
      Integer id = (Integer)o;
      
      String itemCharacterTypeAbbrev = new String("UM"); //default to Unordered Multitype
      for(int i = 0; i < itemCharacterTypes.size(); i++){
	ItemCharacterType ictype = (ItemCharacterType)itemCharacterTypes.elementAt(i);
	if(ictype.getId().intValue() == id.intValue()){
	  itemCharacterTypeAbbrev = ictype.getAbbrev();
	}
      }
      return itemCharacterTypeAbbrev;
    }
    return null;
  }

  public Vector getItemCharacters(){return itemCharacters;}

  public void print(){
    for(int i = 0; i < itemCharacters.size(); i++){
      ItemCharacter ic = (ItemCharacter)itemCharacters.elementAt(i);
      ic.print();
    }
  }

  /**
   * Read in the file
   */
  public void read(DataInputStream dis){
    //read in the file a line at a time
    String line = new String("");

    //We'll put the all the descriptions into a Vector
    Vector descriptions = new Vector();
    Vector description = new Vector();
    int count = 0;
    int descriptionCount = -1;

    boolean beginCharacterList = false;
    boolean endCharacterList = false;
    while(line != null){
      count++;
      try{
	line = dis.readLine();
	//strip out leading white space from lines
	line = eatTrailingSpace(line);

	//ignore blank lines
	if(line != null) {

	  //look for the beginning of the character list
	  int here = line.indexOf("*CHARACTER LIST");
	  if(here  == 0) {
	    beginCharacterList = true;
	  }

	  //flag the beginning of a new directive
	  else if(beginCharacterList && line.indexOf("*") == 0) {
	    endCharacterList = true;
	  }
	  //put all description lines into a Vector for later processing
	  if(beginCharacterList && !endCharacterList){
	    if(isFeatureDescription(line)){
	      //our description counts begin at 0
	      descriptionCount++;
	      description = new Vector();
	      description.addElement(line);
	      descriptions.addElement(description);
	    }
	    else if (descriptionCount > -1){
	      //add the current line to the current description
	      description = (Vector)(descriptions.elementAt(descriptionCount));
	      description.addElement(line);
	    }
	    else {
	      //not a character description
	    }
	  }
	}
      }
      catch(IOException e){
	//System.out.println("Error reading file " + fileName + "at line " + count);
	System.out.println(e);
      }
    }
    System.out.println("done reading file");
    //Make ItemCharacter objects out of the vector of character descriptions
    //and add them to the vector of ItemChars

    System.out.println("Processing " + descriptions.size() + " descriptions");
    for(int i = 0; i < descriptions.size(); i++){
      Vector v = (Vector)descriptions.elementAt(i);
      ItemCharacter ic = makeItemCharacter(v);
      itemCharacters.addElement(ic);
    }
  }

  public void read(String fileName){
    //try to open the file
    DataInputStream dis;
    try{
      dis = new DataInputStream(new FileInputStream(fileName));
    }
    catch(Exception e){
      System.out.println(fileName + " not found");
      return;
    }
    read(dis);
  }


  /**
   * Tests a line to see if it's the beginning of a feature description.
   * Doesn't check if the feature description is valid or not--on if
   * begins with something like "#1. " or "# 1. "
   */
  private boolean isFeatureDescription(String str){
    // check for minimum length
    if(str.length() < 4){
      return false;
    }

    //check for numero (#)
    if(str.charAt(0) != '#') {
      return false;
    }

    //Check for existence of a full stop
    int fullStop = str.indexOf(". ");
    /*for(int i=1; i<fullStop; i++){
      if(!isDigit(str.charAt(i)))
	 //if( !(fullStop == 2 || fullStop == 3) ){
	return false;
	}*/

    int numberBegin = 1;
    //ignore spaces between numero and number
    while(str.charAt(numberBegin) == ' '){
      numberBegin++;
    }
    String numberStr = str.substring(numberBegin, fullStop);
    //see if we have a number or not
    try {
      Integer charNumber = new Integer(numberStr);
    }
    catch(Exception e) {
      System.out.println(e);
      return false;
    }
    return true;
  }
  
  /**
   *  read in a the text of a Delta character description and
   *  construct a DELTA Character.
   */
  public ItemCharacter makeItemCharacter(Vector strings){
    ItemCharacter ic = new ItemCharacter();

    String line = new String();

    //glob all the lines together and adds a blank space between them
    for(int i = 0; i < strings.size(); i++) {
      //replace all tabs with single spaces
      String addition = (String)strings.elementAt(i);
      addition.replace('\t', ' ');
      line = line + addition + " ";
    }

    // Split it up into feature descriptions using the "/ " as a delimiter.
    String delimeter = new String("/ "); //2/12/99, changed to just / by Mike
    NewStringTokenizer st = new NewStringTokenizer(line, delimeter);
    int count = 0;
    String typeAbbrev = new String();
    while(st.hasMoreTokens()){
      count++;
      String feature = st.nextToken();
      feature = eatTrailingSpace(feature);
      // The first token will be the feature description
      if(count == 1){
	parseFeatureDescription(feature, ic);
	if(ic.getId() == null){
	  System.out.print("failed to parse feature description: " + feature);
	  typeAbbrev = "";
	}
	else {
	  // look up the ItemCharacterType of this state
	  typeAbbrev = lookupItemCharacterTypeAbbrev(ic.getId());
	  ic.setType(typeAbbrev);
	}
      }
      else {
	//ic have been set by parseFeatureDescription() above.
	if(ic.getType().equals("UM") || ic.getType().equals("OM")){
	  MultiState s = parseMultiState(feature, typeAbbrev);
	  if(s != null){
	    s.setItemCharacter(ic);
	    ic.addState(s);
	  }
	}
	else if(ic.getType().equals("IN")){
	  NumericState s = new NumericState();
	  s.setIsInteger();
	  s.setName(feature);
	  s.setItemCharacter(ic);
	  ic.addState(s);
	}
	else if(ic.getType().equals("RN")){
	  NumericState s = new NumericState();
	  s.setIsReal();
	  s.setName(feature);
	  s.setItemCharacter(ic);
	  ic.addState(s);
	}
	else { //default to Text state
	  TextState s = new TextState();
	  s.setName(feature);
	  s.setItemCharacter(ic);
	  ic.addState(s);
	}
      }
    }
    return ic;
  }
    
  /**
   * Get the characterid, character name, and comment from the
   * feature description.  
   * Returns a ItemCharacter with the id and name attributes set.
   */
  private ItemCharacter parseFeatureDescription(String feature, ItemCharacter ic){
 
    //Check for position of a full stop
    int fullStop = feature.indexOf(". ");
    if(fullStop == 0){
      return ic;
    }
    //skip past numero
    int numberBegin = 1;
    
    //ignore spaces between numero and number
    while(feature.charAt(numberBegin) == ' '){
      numberBegin++;
    }
    String numberStr = feature.substring(numberBegin, fullStop);
    
    //see if we have a number or not
    try {
     ic.setId(new Integer(numberStr));
    }
    catch(Exception e) {
      System.out.println("Not a valid character number: " + numberStr);
    }

    // look for first description name
    int nameBegin = fullStop + 1;
    int nameEnd = feature.length();
    ic.setFeature(feature.substring(nameBegin, nameEnd));
    return ic;
  }

  public MultiState parseMultiState(String feature, String typeAbbrev) {
    if(feature.length()==0) {
      return null;
    }
    MultiState ms = new MultiState();
    if(typeAbbrev.equals("UM")){
      ms.setUnOrdered();
    }
    else {
      ms.setOrdered();
    }

    //get the Item ID
    int i = 0;
    //this was added on problems reading 3 digit features
    while(isDigit(feature.charAt(i)) && i < feature.length()) {
      i++;
    }
    String numberStr = feature.substring(0, i);
    //make sure if we have a number or not
    Integer id;
    try {
      id = new Integer(numberStr);
    }
    catch(Exception e) {
      id = new Integer(0);
      System.out.println(numberStr + "not a valid ID in feature: " + feature);
    }
    ms.setId(id);

    //move past the '.'
    i++;
    //get the State name
    String nameStr = feature.substring(i, feature.length());

    ms.setName(nameStr);
    return ms;
  }

  private String eatTrailingSpace(String line) {
    if(line == null){
      return line;
    }
    int beginLine;
    int endLine = line.length();
    for(beginLine = 0;  beginLine < endLine; beginLine++){
      if(line.charAt(beginLine) != ' ' && line.charAt(beginLine) != '\t') {
	break;
      }
    }
    return line.substring(beginLine, endLine);
  }  

  private boolean isDigit(char c){
    if(c >= '0' && c <= '9') {
      return true;
    }
    return false;
  }
  public static void main(String args[]) {

    if(args.length != 2){
      System.out.println("Usage: CharsFileReader <chars file name> <specs file name>");
      System.exit(0);
    }
    SpecsFileReader sf = new SpecsFileReader();
    System.out.println("reading specs file...");
    sf.read(args[1]);
    sf.makeItemCharacterTypes();
    System.out.println("done.");

    CharsFileReader cf = new CharsFileReader();
    cf.setItemCharacterTypes(sf.getItemCharacterTypes());
    System.out.println("reading chars file...");
    cf.read(args[0]);
    cf.print();
  }
}
