package mw;
import java.net.*;

/**
 * ImageResource
 * a multimedia display of image, uses IconPanel and ImageFrame to display
 * @author Michael Bartley, based on Image by Noel Cross and Rick Ree
 * 7/16/98
 */


public class ImageResource extends Resource{
  protected String author = null;
  protected String copyrightInfo =  null;

  public ImageResource() {
    mimeType = "image";
  }

  public ImageResource(String fname){
    this(fname, "image");
  }

  public ImageResource(String fname, String mimeType){
    super.setMimeType(mimeType);
    super.setFile(fname);
  }

  public ImageResource(URL location){
    this(location, "image");
  }

  public ImageResource(URL location, String mimeType){
    super(location, mimeType);
  }

  public void display(){
    if(location == null){
      System.out.println("Location of resource not set.");
      return;
    }
    ImageFrame iff = new ImageFrame(location, toString());
    iff.setTitle(super.title);
  }

  public void setAuthor(String s) {author = s;}
  public void setCopyrightInfo(String s) {copyrightInfo = s;}

  public String getAuthor(){return author;}
  public String getCopyrightInfo(){return copyrightInfo;}

  public String toString(){
    StringBuffer sb = new StringBuffer();
    if(super.title != null){
      sb.append(super.title);
    }
    if(author != null){
      sb.append(" by ");
      sb.append(author);
    }
    if(copyrightInfo != null){
      sb.append(".  Copyright: ");
      sb.append(copyrightInfo);
    }
    if(super.notes != null){
      sb.append("  Notes: ");
      sb.append(super.notes);
    }
    sb.append(" ");
    sb.append(super.location.toString());
    return new String(sb);
  }

  public static void main(String arg[]){
    try{
      ImageResource ir = new ImageResource(new URL("http://dev.huh.harvard.edu/~mbartley/images/test.jpg"), "jpeg");
      ir.setTitle("A test");
      ir.setAuthor("Michael Bartley");
      ir.setCopyrightInfo("none");
      ir.setNotes("none");
      ir.display();
    }
    catch(Exception e){
      System.out.println("It ain't workin'" +e);
    }
  }
  
}
