package mw;

import java.awt.*;
import java.util.Vector;

/**
 * SuperPanel.java
 *
 * @author Michael Bartley
 * 3/24/98
 */

class SuperPanel extends Panel{
    ItemCharacterPanel itemCharacterPanel;
    MultiStatePanel    multiStatePanel;
    NumericStatePanel  numericStatePanel;
    ItemsPanel         itemsPanel;
    MyItemPanel        myItemPanel;
    StatePanel         statePanel; //will hold Multi/Numeric StatePanels
    ItemStatePanel     itemStatePanel; //extra popup
    DeltaInterface     delta;
    
    
    //Constructor
    public SuperPanel(DeltaInterface delta){
	super();
	this.delta = delta;
	setBackground(NaviKeyApplet.naviKeyBackgroundColor);
	setForeground(NaviKeyApplet.buttonColor);
	setLayout(new GridLayout(2,2));
	itemCharacterPanel = new ItemCharacterPanel();
	multiStatePanel    = new MultiStatePanel();
	numericStatePanel  = new NumericStatePanel();
	itemsPanel         = new ItemsPanel();
	myItemPanel        = new MyItemPanel(delta);
	statePanel         = new StatePanel();
    }
    
    public void build(){
	statePanel.add("multi",multiStatePanel);
	statePanel.add("numeric", numericStatePanel);
	statePanel.getTheLayout().last(statePanel);
	add(itemCharacterPanel);
	add(statePanel);
	add(myItemPanel);
	add(itemsPanel);
	statePanel.getTheLayout().show(statePanel, "numeric");
    }
    
    public boolean handleEvent(Event e){
	
	//If a character is selected, update the state panel
	if(e.target == itemCharacterPanel.getTheList()){
	    if(e.id == Event.LIST_SELECT){
		
		//get selected character
		BasicItemCharacter ic = itemCharacterPanel.getItemCharacter();
		
		//if it is multi, set StatePanel to MultiState and update
		if(ic.isMultiState()){
		    multiStatePanel.setStates(delta.getStateList(myItemPanel.getStates(), ic.getId(), NaviKeyApplet.intelligence));
		    multiStatePanel.updateList();
		    statePanel.getTheLayout().first(statePanel);
		    repaint();
		}
		
		//if it is numeric set StatePanel to Numeric
		else if(ic.isNumeric()){
		    statePanel.getTheLayout().last(statePanel);
		    repaint();
		}
		//handle text case
		else{
		    System.out.println("I hope this is text.");
		}
	    }
	}
	
	// if a button is clicked in myItemPanel, update items list
	else if(e.target == myItemPanel.getRemoveButton() ||
		e.target == myItemPanel.getResetButton()){
	    if(e.id == Event.ACTION_EVENT){
		myItemPanel.updateList();
		itemsPanel.setItems(delta.getItemList(myItemPanel.getStates()));
		itemsPanel.updateList();
		// Useful for shrinking/growing character lists, 
		// uncomment next two lines once implemented
		//COMMENTED OUT FOR SPEED
		//MIKE, 3/9/99
		if(NaviKeyApplet.intelligence){
		    itemCharacterPanel.setItemCharacters(delta.getItemCharacterList(myItemPanel.getStates(), NaviKeyApplet.intelligence));
		    itemCharacterPanel.updateList();
		}
		//Blank out state panel
		blankOutStatePanel();
		//repaint();
	    }
	}
	
	//	else if(e.target == myItemPanel.getSmartsCheckbox()){
	//  if(e.id == Event.ACTION_EVENT){
	//boolean old = ((Checkbox)e.target).getState();
	//boolean newb = (old ? true : false);
	//NaviKeyApplet.intelligence = newb;
	////System.out.println("Now smarts are "+NaviKeyApplet.intelligence);
	////tmp.setState(newb);
	//  }
	//}//

	// if a State is selected update it in MyItem
	else if(e.target == multiStatePanel.getTheList()){
	    if(e.id == Event.LIST_SELECT){
		myItemPanel.addState(multiStatePanel.getState());
		myItemPanel.updateList();
		
		itemsPanel.setItems(delta.getItemList(myItemPanel.getStates()));
		itemsPanel.updateList();
		
		// Useful for shrinking/growing character lists, 
		//COMMENTED OUT FOR SPEED
		//MIKE, 3/9/99
		if(NaviKeyApplet.intelligence){
		    itemCharacterPanel.setItemCharacters(delta.getItemCharacterList(myItemPanel.getStates(), NaviKeyApplet.intelligence));
		    itemCharacterPanel.updateList();
		}
		
		blankOutStatePanel();
		//repaint();
	    }
	}
	
	// pop-up list of states defining selected item
	else if(e.target == itemsPanel.getTheList()){
	    if(e.id == Event.LIST_SELECT){
		Object o = itemsPanel.getItem().getId();
		String title = delta.getItemTitle(o);
		Vector strings = delta.getItemDescription(o);
		Vector resources = delta.getItemResources(o);
		ItemStatePanel isp = new ItemStatePanel(title,strings,resources);
	    }
	}
	
	// handle numeric state information addition
	else if(e.target == numericStatePanel.getAdd()){
	    if(e.id == Event.ACTION_EVENT){
		BasicState ns = numericStatePanel.getState();
		ns.setCharacterId(itemCharacterPanel.getItemCharacter().getId());
		
		myItemPanel.addState(ns);
		myItemPanel.updateList();
		
		itemsPanel.setItems(delta.getItemList(myItemPanel.getStates()));
		itemsPanel.updateList();
		
		// Useful for shrinking/growing character lists, 
		// uncomment next two lines once implemented
		//COMMENTED OUT FOR SPEED
		//MIKE, 3/9/99
		if(NaviKeyApplet.intelligence){
		    itemCharacterPanel.setItemCharacters(delta.getItemCharacterList(myItemPanel.getStates(), NaviKeyApplet.intelligence));
		    itemCharacterPanel.updateList();
		}
		
		blankOutStatePanel();
		//repaint();
	    }
	}
	return super.handleEvent(e);
    }
    
    private void 	blankOutStatePanel(){
	numericStatePanel.reset();
	multiStatePanel.setStates(new Vector());
	multiStatePanel.updateList();
	statePanel.getTheLayout().first(statePanel);
    }
    
}
